
Field_Bin = {};

function Field_Bin.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations)
       and SpecializationUtil.hasSpecialization(Trailer, specializations)
       and SpecializationUtil.hasSpecialization(ImplementLinks, specializations)
       and SpecializationUtil.hasSpecialization(manuallyPowerShaft, specializations);
end;

function Field_Bin:load(xmlFile)
    self.setUnloadingState = SpecializationUtil.callSpecializationsFunction("setUnloadingState");
    self.setVehicleRpmUp = SpecializationUtil.callSpecializationsFunction("setVehicleRpmUp");
    self.trailerRaycastCallback = Field_Bin.trailerRaycastCallback;

    self.triggerNode = {};
    self.triggerNode.index = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.unloadingTrigger#node"));
    self.triggerNode.distance = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.unloadingTrigger#maxDistance"), 1);
    
    self.pipe = {};
    self.pipe.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.pipe#index"));
    self.pipe.distance = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.pipe#raycastDistance"), 7);
    --self.pipe.out = false;
    
    self.unloadingCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.unloadingCapacity"), 100);
    self.saveMinRpm = nil;
    self.allowFillFromAir = false;
    self.isUnloading = false;
    self.allowUnloading = false;
    self.notAcceptedFruitType = 0;

    local path = Utils.getFilename("Hud/overlay.dds", self.baseDirectory);
    self.unloadingOverlay = Overlay:new("hudPDAControl", path, g_currentMission.fruitSymbolX, g_currentMission.fruitSymbolY-0.11, g_currentMission.fruitSymbolSize, g_currentMission.fruitSymbolSize * (4 / 3)); 

    self.setDoorOne = SpecializationUtil.callSpecializationsFunction("setDoorOne");
    self.DoorOneAnimation = getXMLString(xmlFile, "vehicle.DoorOne#animationName");
    self.isDoorOneOn = false;
    
    self.setDoorTwo = SpecializationUtil.callSpecializationsFunction("setDoorTwo");
    self.DoorTwoAnimation = getXMLString(xmlFile, "vehicle.DoorTwo#animationName");
    self.isDoorTwoOn = false;
    
    self.setWheelRot = SpecializationUtil.callSpecializationsFunction("setWheelRot");
    self.WheelRotAnimation = getXMLString(xmlFile, "vehicle.WheelRot#animationName");
    self.isWheelRotOn = false;
    
    self.binNode = {};
    self.binNode.node  = Utils.indexToObject(self.components,getXMLString(xmlFile, "vehicle.binNode#index"));
    
    self.newPtoSet = false;
    self.newPtoVehicle = nil;
    
    self.printWarningTime = 0;
end;

function Field_Bin:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    if not resetVehicles then
        local isWheelRotOn = Utils.getNoNil(getXMLBool(xmlFile, key .. "#isWheelRotOn"), false);
        if isWheelRotOn ~= nil then
            self:setWheelRot(isWheelRotOn);
        end;
    end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function Field_Bin:getSaveAttributesAndNodes(nodeIdent)
    local attributes = ' ';

    local mystring = 'isWheelRotOn="' .. tostring(self.isWheelRotOn) ..'"';    
    attributes = attributes .. mystring;

    local node = nil;
    return attributes, node;
end;

function Field_Bin:delete()
    delete(self.unloadingOverlay);
    self.unloadingOverlay = nil;
end;

function Field_Bin:readStream(streamId, connection)
    self.notAcceptedFruitType   = streamReadUIntN(streamId, Fillable.sendNumBits);
    self.isUnloading            = streamReadBool(streamId);
    self.allowUnloading         = streamReadBool(streamId);
    self:setWheelRot(streamReadBool(streamId), true); 
    self:setDoorOne(streamReadBool(streamId), true);
    self:setDoorTwo(streamReadBool(streamId), true);
end;

function Field_Bin:writeStream(streamId, connection)
    streamWriteUIntN(streamId, self.notAcceptedFruitType, Fillable.sendNumBits);
    streamWriteBool(streamId, self.isUnloading);
    streamWriteBool(streamId, self.allowUnloading);
    streamWriteBool(streamId, self.isWheelRotOn);
    streamWriteBool(streamId, self.isDoorOneOn);
    streamWriteBool(streamId, self.isDoorTwoOn);
end;

-- Helper function, because checking "not self.PTOId" is unintuitive.
function Field_Bin:hasPtoAttached()
    return (not self.PTOId);    -- ImplementLinks.LUA
end

-- Helper function, to get attacher-vehicle even if it is only attached by the PTO.
function Field_Bin:getAttacherVehicle()
    local vehicle = self.attacherVehicle;
    if vehicle == nil and self.ptoShaft ~= nil then
        vehicle = self.ptoShaft.vehicle; -- manuallyPowerShaft.LUA
    end
    return vehicle;
end;

function Field_Bin:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Field_Bin:keyEvent(unicode, sym, modifier, isDown)
end;

function Field_Bin:update(dt)
    -- Server only
    if self.isServer then
        -- If any door is open(true), then allow fill from air (even into the closed one).
        self.allowFillFromAir = self.isDoorOneOn or self.isDoorTwoOn;
        
        for i, jointDesc in pairs(self.componentJoints) do
            setJointFrame(self.componentJoints[i].jointIndex, 0, self.componentJoints[i].jointNode);
        end;
    end;

    -- Client
    if self.binInRange then
        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
            self:setDoorOne(not self.isDoorOneOn);
        end;
        if self.isDoorOneOn then
            g_currentMission:addHelpButtonText(string.format(g_i18n:getText("Close_DoorOne"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA3);
        else
            g_currentMission:addHelpButtonText(string.format(g_i18n:getText("Open_DoorOne"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA3);
        end;
        
        if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
            self:setDoorTwo(not self.isDoorTwoOn);
        end;
        if self.isDoorTwoOn then
            g_currentMission:addHelpButtonText(string.format(g_i18n:getText("Close_DoorTwo"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
        else
            g_currentMission:addHelpButtonText(string.format(g_i18n:getText("Open_DoorTwo"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
        end;    
        
        if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) and self.fillLevel <= 0 then
            self:setWheelRot(not self.isWheelRotOn);
        end;
        if self.fillLevel <= 0 then
            if self.isWheelRotOn then
                g_currentMission:addHelpButtonText(string.format(g_i18n:getText("field"), self.typeDesc), InputBinding.LOWER_IMPLEMENT);
            else
                g_currentMission:addHelpButtonText(string.format(g_i18n:getText("transport"), self.typeDesc), InputBinding.LOWER_IMPLEMENT);
            end;
        end;
    end;

    if self:getIsActiveForInput() or (self.ptoShaft.vehicle ~= nil and self.ptoShaft.vehicle:getIsActiveForInput(false)) then
        if self.notAcceptedFruitType ~= 0 then
            if FruitUtil.fruitIndexToDesc[self.notAcceptedFruitType].name ~= nil then
                g_currentMission:addWarning(g_i18n:getText(FruitUtil.fruitIndexToDesc[self.notAcceptedFruitType].name) .. g_i18n:getText("notAcceptedHere"), 0.018, 0.033);
            end;
        else
            if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
                if Field_Bin.hasPtoAttached(self) then
                    self:setUnloadingState(not self.isUnloading);
                else
                    self.printWarningTime = self.time + 1000;
                end;
            end;

            if self.isUnloading then
                g_currentMission:addHelpButtonText(g_i18n:getText("DeactivateAuger"), InputBinding.IMPLEMENT_EXTRA);
            else
                g_currentMission:addHelpButtonText(g_i18n:getText("ActivateAuger"), InputBinding.IMPLEMENT_EXTRA);
            end;
            
            if self.printWarningTime > self.time then
                g_currentMission:addWarning(g_i18n:getText("turnON_Error"), 0.018, 0.033);
            end;
        end;
    end;
end;

function Field_Bin:updateTick(dt)
    -- Server only
    if self.isServer then
        -- TODO: try to use Satis' 'Work Throttle' instead for this.
        if self.ptoShaft.vehicle ~= nil then
            if not self.newPtoSet then
                self.saveMinRpm = self.ptoShaft.vehicle.motor.minRpm;
                self.newPtoVehicle = self.ptoShaft.vehicle;
                self.newPtoSet = true;
            end;
        else
            if self.newPtoSet then
                if self.attacherVehicle == nil then
                    self.newPtoVehicle.motor.minRpm = self.saveMinRpm;
                    self.newPtoVehicle = nil;
                    self.saveMinRpm = nil;
                end;
                self.newPtoSet = false;
            end;
        end;

        --        
        self.trailerFoundId = nil;
        local unloadingTipTrigger = nil;
        
        if self.fillLevel > 0 then
            -- force lift wheels if filled
            if self.isWheelRotOn then
                self:setWheelRot(false);
            end;
            
            for k, tipTrigger in pairs(g_currentMission.tipTriggers) do
                local trailerX, trailerY, trailerZ = getWorldTranslation(self.triggerNode.index);
                local triggerX, triggerY, triggerZ = getWorldTranslation(tipTrigger.triggerId);
                local distance = Utils.vector3Length(trailerX-triggerX, trailerY-triggerY, trailerZ-triggerZ);
                if distance < self.triggerNode.distance then
                    unloadingTipTrigger = tipTrigger;
                end;
            end;
            if unloadingTipTrigger == nil then
                -- Not near a tip-trigger, so try to find if a trailer is under the pipe/auger.
                local x, y, z = getWorldTranslation(self.pipe.node);
                local dx, dy, dz = localDirectionToWorld(self.pipe.node, 0, -0.5, 0);
                raycastAll(x, y, z, dx, dy, dz, "trailerRaycastCallback", self.pipe.distance, self);
            end;
        end;
        
        local notAcceptedFruitType = 0;
        local allowUnloading = false;
        local isUnloading = self.isUnloading;
        local deltaLevel = 0;
        
        local vehicle = Field_Bin.getAttacherVehicle(self);
        if vehicle ~= nil then
            if vehicle.isMotorStarted and Field_Bin.hasPtoAttached(self) then
                if self.trailerFoundId ~= nil then
                    local trailer = g_currentMission.nodeToVehicle[self.trailerFoundId];
                    if trailer ~= nil and trailer ~= self and trailer:allowFillType(self.currentFillType, true) and trailer.allowFillFromAir and trailer.capacity > trailer.fillLevel then
                        allowUnloading = true;
                        if isUnloading then
                            deltaLevel = math.min(self.unloadingCapacity*dt/1000.0, trailer.capacity-trailer.fillLevel);
                            trailer:setFillLevel(trailer.fillLevel+deltaLevel, self.currentFillType);
                        end;
                    end;
                elseif unloadingTipTrigger ~= nil then
                    local tipTrigger = unloadingTipTrigger;
                    local fruitType = FruitUtil.fillTypeToFruitType[self.currentFillType];
                    local fruitAccept = tipTrigger.acceptedFruitTypes[fruitType];
                    if not fruitAccept then
                        notAcceptedFruitType = fruitType;
                    elseif tipTrigger.isFarmTrigger then
                        allowUnloading = true;
                        if isUnloading then
                            deltaLevel = self.unloadingCapacity*dt/1000.0;
                            tipTrigger:updateTrailerTipping(self, -deltaLevel, self.currentFillType);
                        end;
                    end;
                end;
            end;
        end;
        --
        local newFillLevel = self.fillLevel - deltaLevel;
        if newFillLevel <= 0 then
            -- FieldBin has become empty
            isUnloading = false;
            self:setFillLevel(0, self.currentFillType);
        elseif deltaLevel <= 0 then
            -- FieldBin can not unload more, or auger is stopped
            isUnloading = false;
        elseif isUnloading then
            -- FieldBin is (still) unloading, so set its new fill-level
            self:setFillLevel(newFillLevel, self.currentFillType);
        end;

        --
        if self.allowUnloading       ~= allowUnloading
        or self.notAcceptedFruitType ~= notAcceptedFruitType
        or self.isUnloading          ~= isUnloading then
            self.allowUnloading         = allowUnloading;
            self.notAcceptedFruitType   = notAcceptedFruitType;
            self.isUnloading            = isUnloading;
            --
            FieldBinUnloadingEvent.sendEvent(self, isUnloading, allowUnloading, notAcceptedFruitType);
        end;
    end;

    -- Client
    if self.isClient then
        -- Particle effect, when unloading.
        if self.isUnloading then
            if self.particleFillType ~= nil and self.particleFillType ~= self.currentFillType then
                Utils.setEmittingState(self.tipAnimations[1].dischargeParticleSystems[self.particleFillType], false);
            end
            self.particleFillType = self.currentFillType;
            Utils.setEmittingState(self.tipAnimations[1].dischargeParticleSystems[self.particleFillType], true);
        elseif self.particleFillType then
            Utils.setEmittingState(self.tipAnimations[1].dischargeParticleSystems[self.particleFillType], false);
            self.particleFillType = nil;
        end;
        
        -- Determine if player, that is on foot, is within range of the tool, which in that case causes extra functions/helpbox texts to appear 
        if g_currentMission.player ~= nil and g_currentMission.controlPlayer then
            local nearestDistance = 6;
            local vx, vy, vz = getWorldTranslation(g_currentMission.player.rootNode);        
            local px, py, pz = getWorldTranslation(self.binNode.node); 
            local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);    
            if distance < nearestDistance then
                self.binInRange = true; 
            else
                self.binInRange = false; 
            end;
        else
            self.binInRange = false; 
        end;
    end;

    --
    -- TODO: try to use Satis' 'Work Throttle' instead for this.
    self:setVehicleRpmUp(dt, self.isUnloading);    
end;

function Field_Bin:trailerRaycastCallback(transformId, x, y, z, distance)
    local vehicle = g_currentMission.nodeToVehicle[transformId];
    if vehicle ~= nil then
        if vehicle.exactFillRootNode == transformId then
            self.trailerFoundId = transformId;
            return false;
        end;
    end;
    return true;
end;

function Field_Bin:setUnloadingState(state, noEventSend)
    FieldBinUnloadingEvent.sendEvent(self, state, self.allowUnloading, self.notAcceptedFruitType, noEventSend);
    self.isUnloading = state;
end;

function Field_Bin:setVehicleRpmUp(dt, isActive)
    local vehicle = Field_Bin.getAttacherVehicle(self);
    if vehicle ~= nil and self.saveMinRpm ~= nil then
        -- TODO: try to use Satis' 'Work Throttle' instead for this.
        if dt ~= nil then
            if isActive == true then
                vehicle.motor.minRpm = math.max(vehicle.motor.minRpm-(dt*2), -1200);
            else
                vehicle.motor.minRpm = math.min(vehicle.motor.minRpm+(dt*5), self.saveMinRpm);
            end;
        else
            vehicle.motor.minRpm = self.saveMinRpm;
        end;

        if self.isServer then
            if vehicle.isMotorStarted then
                local fuelUsed = 0.0000011*math.abs(vehicle.motor.minRpm);
                vehicle:setFuelFillLevel(vehicle.fuelFillLevel-fuelUsed);
                g_currentMission.missionStats.fuelUsageTotal = g_currentMission.missionStats.fuelUsageTotal + fuelUsed;
                g_currentMission.missionStats.fuelUsageSession = g_currentMission.missionStats.fuelUsageSession + fuelUsed;
            end;
        end;
    end;
end;

function Field_Bin:draw()
    if self.isClient then
        if not self.isWheelRotOn and self.lastSpeed*3600 > 2 then
            if self.fillLevel == 0 then
                g_currentMission:addWarning(g_i18n:getText("moving_Error2"), 0.018, 0.033);
            else
                g_currentMission:addWarning(g_i18n:getText("moving_Error"), 0.018, 0.033);
            end;
        end;
        if self.allowUnloading then
            self.unloadingOverlay:render();
        end
    end;
end;

function Field_Bin:onAttach(attacherVehicle)
    if self.attacherVehicleCopy == nil then
        self.attacherVehicleCopy = self.attacherVehicle;
    end;
    if self.attacherVehicle.motor ~= nil then
        self.saveMinRpm = self.attacherVehicle.motor.minRpm;
    end;
end;

function Field_Bin:onDetach()
    if self.deactivateOnDetach then
        self:onDeactivate(self);
    else
        self:onDeactivateSounds(self);
    end;
    --
    if self.saveMinRpm ~= nil then
        for k, steerable in pairs(g_currentMission.steerables) do
            if self.attacherVehicleCopy == steerable then
                steerable.motor.minRpm = self.saveMinRpm;
                self.attacherVehicleCopy = nil;
                self.saveMinRpm = nil;
            end;
        end;
    end;
    --
    self:setWheelRot(false,true);
end;

function Field_Bin:onLeave()
    if self.deactivateOnLeave then
        self:onDeactivate(self);
    else
        self:onDeactivateSounds(self);
    end;
end;

function Field_Bin:onDeactivate()
    self:setUnloadingState(false, true);
    self:setVehicleRpmUp(nil, false);
    for k, particle in pairs(self.tipAnimations[1].dischargeParticleSystems) do
        Utils.setEmittingState(particle, false);
    end;
end;

function Field_Bin:onDeactivateSounds()
end;

function Field_Bin:setWheelRot(newState,noEventSend)
    FieldBinEvent.sendEvent(self, newState, self.isDoorOneOn, self.isDoorTwoOn, noEventSend);
    --
    if self.isWheelRotOn ~= newState then
        self.isWheelRotOn = newState;
        -- Play WheelRot animation --
        if self.isWheelRotOn then
            if self.WheelRotAnimation ~= nil and self.playAnimation ~= nil then
                self:playAnimation(self.WheelRotAnimation, 1, nil, true);
            end;
        else
            if self.WheelRotAnimation ~= nil and self.playAnimation ~= nil then
                self:playAnimation(self.WheelRotAnimation, -1, nil, true);
            end;
        end;
    end;
end;

function Field_Bin:setDoorOne(newState,noEventSend)
    FieldBinEvent.sendEvent(self, self.isWheelRotOn, newState, self.isDoorTwoOn, noEventSend);
    --
    if self.isDoorOneOn ~= newState then
        self.isDoorOneOn = newState;
        -- Play DoorOne animation --
        if self.isDoorOneOn then
            if self.DoorOneAnimation ~= nil and self.playAnimation ~= nil then
                self:playAnimation(self.DoorOneAnimation, 1, nil, true);
            end;
        else
            if self.DoorOneAnimation ~= nil and self.playAnimation ~= nil then
                self:playAnimation(self.DoorOneAnimation, -1, nil, true);
            end;
        end;    
    end;
end;

function Field_Bin:setDoorTwo(newState,noEventSend)
    FieldBinEvent.sendEvent(self, self.isWheelRotOn, self.isDoorOneOn, newState, noEventSend);
    --
    if self.isDoorTwoOn ~= newState then
        self.isDoorTwoOn = newState;
        -- Play DoorTwo animation --
        if self.isDoorTwoOn then
            if self.DoorTwoAnimation ~= nil and self.playAnimation ~= nil then
                self:playAnimation(self.DoorTwoAnimation, 1, nil, true);
            end;
        else
            if self.DoorTwoAnimation ~= nil and self.playAnimation ~= nil then
                self:playAnimation(self.DoorTwoAnimation, -1, nil, true);
            end;
        end;
    end;
end;

--
--
--

FieldBinEvent = {};
FieldBinEvent_mt = Class(FieldBinEvent, Event);

InitEventClass(FieldBinEvent, "FieldBinEvent");

function FieldBinEvent:emptyNew()
    local self = Event:new(FieldBinEvent_mt);
    self.className="FieldBinEvent";
    return self;
end;

function FieldBinEvent:new(vehicle, isWheelRot, isDoorOne, isDoorTwo)
    local self = FieldBinEvent:emptyNew()
    self.vehicle    = vehicle;
    self.isWheelRot = isWheelRot;
    self.isDoorOne  = isDoorOne;
    self.isDoorTwo  = isDoorTwo;    
    return self;
end;

function FieldBinEvent:readStream(streamId, connection)
    self.vehicle    = networkGetObject(streamReadInt32(streamId));
    self.isWheelRot = streamReadBool(streamId);
    self.isDoorOne  = streamReadBool(streamId);
    self.isDoorTwo  = streamReadBool(streamId);
    self:run(connection);
end;

function FieldBinEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
    streamWriteBool(streamId, self.isWheelRot);
    streamWriteBool(streamId, self.isDoorOne);
    streamWriteBool(streamId, self.isDoorTwo);
end;

function FieldBinEvent:run(connection)   
    if self.vehicle ~= nil then
        self.vehicle:setWheelRot(self.isWheelRot, true);
        self.vehicle:setDoorOne(self.isDoorOne, true);
        self.vehicle:setDoorTwo(self.isDoorTwo, true);
        if not connection:getIsServer() then
            g_server:broadcastEvent(FieldBinEvent:new(self.vehicle, self.isWheelRot, self.isDoorOne, self.isDoorTwo), nil, connection, self.vehicle);
        end;
    end;
end;

function FieldBinEvent.sendEvent(vehicle, isWheelRot, isDoorOne, isDoorTwo, noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(FieldBinEvent:new(vehicle, isWheelRot, isDoorOne, isDoorTwo), nil, nil, vehicle);
        else
            g_client:getServerConnection():sendEvent(FieldBinEvent:new(vehicle, isWheelRot, isDoorOne, isDoorTwo));
        end;
    end;
end;

--
--
--

FieldBinUnloadingEvent = {};
FieldBinUnloadingEvent_mt = Class(FieldBinUnloadingEvent, Event);

InitEventClass(FieldBinUnloadingEvent, "FieldBinUnloadingEvent");

function FieldBinUnloadingEvent:emptyNew()
    local self = Event:new(FieldBinUnloadingEvent_mt);
    self.className="FieldBinUnloadingEvent";
    return self;
end;

function FieldBinUnloadingEvent:new(vehicle, isUnloading, allowUnloading, notAcceptedFruitType)
    local self = FieldBinUnloadingEvent:emptyNew()
    self.vehicle                = vehicle;
    self.isUnloading            = isUnloading;
    self.allowUnloading         = allowUnloading;
    self.notAcceptedFruitType   = notAcceptedFruitType;
    return self;
end;

function FieldBinUnloadingEvent:readStream(streamId, connection)
    self.vehicle                = networkGetObject(streamReadInt32(streamId));
    self.notAcceptedFruitType   = streamReadUIntN(streamId, Fillable.sendNumBits);
    self.isUnloading            = streamReadBool(streamId);
    self.allowUnloading         = streamReadBool(streamId);
    self:run(connection);
end;

function FieldBinUnloadingEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
    streamWriteUIntN(streamId, self.notAcceptedFruitType, Fillable.sendNumBits);
    streamWriteBool(streamId, self.isUnloading);
    streamWriteBool(streamId, self.allowUnloading);
end;

function FieldBinUnloadingEvent:run(connection)   
    if self.vehicle ~= nil then
        if connection:getIsServer() then
            -- At a client, which got server message.
            self.vehicle.isUnloading            = self.isUnloading;
            self.vehicle.allowUnloading         = self.allowUnloading;
            self.vehicle.notAcceptedFruitType   = self.notAcceptedFruitType;
        else
            -- At the server, which got a client instruction to start/stop unloading.
            self.vehicle:setUnloadingState(self.isUnloading);
        end;
    end;
end;

function FieldBinUnloadingEvent.sendEvent(vehicle, isUnloading, allowUnloading, notAcceptedFruitType, noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(FieldBinUnloadingEvent:new(vehicle, isUnloading, allowUnloading, notAcceptedFruitType), nil, nil, vehicle);
        else
            g_client:getServerConnection():sendEvent(FieldBinUnloadingEvent:new(vehicle, isUnloading, allowUnloading, notAcceptedFruitType));
        end;
    end;
end;
